//
//     Generated by class-dump 3.5 (64 bit).
//
//     class-dump is Copyright (C) 1997-1998, 2000-2001, 2004-2013 by Steve Nygard.
//

#import <UIKit/UITapGestureRecognizer.h>

@interface UITapGestureRecognizer (RecordingAdditions)
- (id)_automationName;
@end
