//
//     Generated by class-dump 3.5 (64 bit).
//
//     class-dump is Copyright (C) 1997-1998, 2000-2001, 2004-2013 by Steve Nygard.
//

@class NSString;

@interface XCUIRecorderTimingMessage : NSObject
{
    double _start;
    NSString *_message;
}
@property(copy) NSString *message; // @synthesize message=_message;
@property double start; // @synthesize start=_start;

+ (id)descriptionForTimingMessages:(id)arg1;
+ (id)messageWithString:(id)arg1;

@end
